from faker import Faker

from .models.items import Item


class DummyItemsGenerator:
    def __init__(self, num_items: int):
        self.num_items = num_items
        self.fake = Faker()
        self.names = [
            "Apple",
            "Pear",
            "Lemon",
            "Chair",
            "Window",
            "Door",
            "Computer",
            "Fridge",
        ]
        self.items = []

    def create_items(self):
        for i in range(self.num_items):
            item_attrs = {}
            item_attrs["id"] = i
            item_attrs[
                "name"
            ] = f"{self.fake.color_name()} {self.fake.word(ext_word_list=self.names)}"
            item_attrs["price"] = self.fake.pyfloat(
                min_value=1, max_value=500, right_digits=2
            )
            item_attrs["tax"] = self.fake.pyfloat(
                min_value=0, max_value=0.35, right_digits=2
            )
            item_attrs["description"] = self.fake.sentence(nb_words=5)
            self.items.append(Item(**item_attrs))

        return self.items



try:
    import sphinx.cmd.build as sphinx_build
    import sphinx.ext.apidoc as sphinx_apidoc
except ImportError as exc:
    print("dev dependencies must be installed to generate documentation")


def main():
    sphinx_apidoc.main(["-o", "docs/source/gen", "project"])
    sphinx_build.main(["docs/source", "docs/build"])


if __name__ == "__main__":
    main()



import uvicorn

from os import getenv
from .app import app
from .dummy_data import DummyItemsGenerator
from .exceptions import ItemNotFoundException
from .models.items import Item, PatchedUserItem, UserItem

"""Dummy data"""
dummy_items_gen = DummyItemsGenerator(num_items=100)
items = dummy_items_gen.create_items()


"""Helper functions"""


def check_if_item_exists(id: int) -> bool:
    ids = [item.id for item in items]
    return True if id in ids else False


def check_if_new_id_valid(id: int):
    return id == len(items)


"""HTTP methods"""


@app.get("/", response_model=dict)
async def root():
    """Returns a basic hello message."""
    return {"message": "Hello, ASH!"}


@app.get("/items/", response_model=list[Item])
async def get_items():
    """Returns all items."""
    return items


@app.post("/items/", response_model=Item)
async def post_item(user_item: UserItem):
    """Allows user to add an `Item` to the items list."""
    new_id = len(items)
    new_item = Item(id=new_id, **user_item.dict())
    items.append(new_item)

    return new_item


@app.get("/items/{id}", response_model=Item)
async def get_item(id: int):
    """Returns an `Item` based on `ID`, if `ID` is valid and exists; else, raises `ItemNotFoundException`."""
    if check_if_item_exists(id) == False:
        raise ItemNotFoundException(id)

    return items[id]


@app.patch("/items/{id}", response_model=Item)
async def patch_item(id: int, patches: PatchedUserItem):
    """Allows user to patch item, if `ID` is valid and exists; else, raises `ItemNotFoundException`."""
    if check_if_item_exists(id) == False:
        raise ItemNotFoundException(id)

    stored_item = items[id]
    updated_item = stored_item.copy(update=patches.dict(exclude_unset=True))
    items[id] = updated_item

    return updated_item


def main():
    uvicorn.run(app, host="127.0.0.1", port=8000)


if __name__ == "__main__":
    main()



from fastapi import Request
from fastapi.responses import JSONResponse

from .app import app


class ItemNotFoundException(Exception):
    def __init__(self, name):
        self.name = name


@app.exception_handler(ItemNotFoundException)
async def item_not_found_exception_handler(
    request: Request, exc: ItemNotFoundException
):
    return JSONResponse(
        status_code=404,
        content={"message": f"Item with ID [{exc.name}] not found or invalid"},
    )


class NewItemIDNotValidException(Exception):
    def __init__(self, name):
        self.name = name


@app.exception_handler(NewItemIDNotValidException)
async def new_item_id_not_valid_exception_handler(
    request: Request, exc: ItemNotFoundException
):
    return JSONResponse(status_code=406, content={"message": f"Invalid item ID."})



from typing import Optional

from pydantic import BaseModel


class Item(BaseModel):
    """The format used by the server logic."""

    id: int
    name: str
    description: Optional[str] = None
    price: float
    tax: Optional[float] = None


class UserItem(BaseModel):
    """The format for creating a new item by the user."""

    name: str
    description: Optional[str] = None
    price: float
    tax: Optional[float] = None


class PatchedUserItem(BaseModel):
    """The format for patching an existing item."""

    name: Optional[str] = None
    description: Optional[str] = None
    price: Optional[float] = None
    tax: Optional[float] = None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_example_server', 'fastapi_example_server.models']

package_data = \
{'': ['*'],
 'fastapi_example_server': ['.pytest_cache/*', '.pytest_cache/v/cache/*']}

install_requires = \
['Faker>=14.2.0,<15.0.0', 'fastapi[all]>=0.81.0,<0.82.0']

entry_points = \
{'console_scripts': ['doc = fastapi_example_server.doc:main',
                     'main = fastapi_example_server.main:main']}

setup_kwargs = {
    'name': 'fastapi-example-server',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Julian Lechner',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10',
}


setup(**setup_kwargs)
